@@ -2,42 +2,17 @@
 """elementfactory.py
 """
 
+from zope import interface
+from zope import component
 from gaphor.misc import uniqueid, odict
 from gaphor.undomanager import get_undo_manager
 from gaphor.UML.element import Element
 from gaphor.UML.diagram import Diagram
-
-class _UndoCreateAction(object):
-
-    def __init__(self, factory, element):
-        self.factory = factory
-        self.element = element
-
-    def undo(self):
-        try:
-            del self.factory._elements[self.element.id]
-        except KeyError:
-            pass # Key was probably already removed in an unlink call
-        self.factory.notify(self.element, 'remove')
-
-    def redo(self):
-        self.factory._elements[self.element.id] = self.element
-        self.factory.notify(self.element, 'create')
-
-
-class _UndoRemoveAction(object):
-
-    def __init__(self, factory, element):
-        self.factory = factory
-        self.element = element
-
-    def undo(self):
-        self.factory._elements[self.element.id] = self.element
-        self.factory.notify(self.element, 'create')
-
-    def redo(self):
-        del self.factory._elements[self.element.id]
-        self.factory.notify(self.element, 'remove')
+from gaphor.UML.interfaces import IElementCreateEvent, IElementDeleteEvent, \
+                                  IFlushFactoryEvent, IModelFactoryEvent, \
+                                  IService
+from gaphor.UML.event import ElementCreateEvent, ElementDeleteEvent, \
+                             FlushFactoryEvent, ModelFactoryEvent
 
 
 class ElementFactory(object):
@@ -52,6 +27,8 @@ class ElementFactory(object):
     flush - model is flushed: all element are removed from the factory
             (element is None)
     """
+    interface.implements(IService)
+
     def __init__(self):
         self._elements = odict.odict()
         self._observers = list()
@@ -68,9 +45,10 @@ class ElementFactory(object):
         assert issubclass(type, Element)
         obj = type(id, self)
         self._elements[id] = obj
-        get_undo_manager().add_undo_action(_UndoCreateAction(self, obj))
-        obj.connect('__unlink__', self.__element_signal)
+        #get_undo_manager().add_undo_action(_UndoCreateAction(self, obj))
+        obj.connect('__unlink__', self._element_signal)
         self.notify(obj, 'create')
+        component.handle(ElementCreateEvent(self, obj))
         return obj
 
     def size(self):
@@ -86,15 +64,21 @@ class ElementFactory(object):
     __getitem__ = lookup
 
     def select(self, expression=None):
-        """Create a list of elements that comply with expression.
+        """Iterate elements that comply with expression.
         """
         if expression is None:
-            return self.values()
-        l = []
-        for e in self._elements.itervalues():
-            if expression(e):
-                l.append(e)
-        return l
+            for e in self._elements.itervalues():
+                yield e
+        else:
+            for e in self._elements.itervalues():
+                if expression(e):
+                    yield e #l.append(e)
+        #return l
+
+    def lselect(self, expression=None):
+        """Like select(), but returns a list.
+        """
+        return list(self.select(expression))
 
     def keys(self):
         """Return a list with all id's in the factory.
@@ -126,8 +110,12 @@ class ElementFactory(object):
     def flush(self):
         """Flush all elements (remove them from the factory)."""
         self.notify(None, 'flush')
+        component.handle(FlushFactoryEvent(self))
+
         # First flush all diagrams:
-        for value in self.select(lambda e: isinstance(e, Diagram)):
+        for value in list(self.select(lambda e: isinstance(e, Diagram))):
+            # Make sure no updates happen while destroying the canvas
+            value.canvas.block_updates = True
             value.unlink()
 
         for key, value in self._elements.items():
@@ -137,9 +125,16 @@ class ElementFactory(object):
 
         assert len(self._elements) == 0, 'Still items in the factory: %s' % str(self._elements.values())
 
+        # Force Garbage collection, so memory allocated by items is freed.
         import gc
         for i in range(4): gc.collect()
 
+    def swap_element(self, element, new_class):
+	assert element in self._elements.values()
+        if element.__class__ is not new_class:
+            element.__class__ = new_class
+            element.notify('__class__')
+
     def connect(self, callback, *data):
         """Attach 'callback'."""
         self._observers.append((callback,) + data)
@@ -167,18 +162,53 @@ class ElementFactory(object):
 
     def notify_model(self):
         self.notify(None, 'model')
+        component.handle(ModelFactoryEvent(self))
 
-    def __element_signal(self, element, pspec):
+    def _element_signal(self, element, pspec):
         """Remove an element from the factory """
         #log.debug('element %s send signal %s' % (element, name))
         if pspec == '__unlink__' and self._elements.has_key(element.id):
             #log.debug('Unlinking element: %s' % element)
             # TODO: make undo action
             del self._elements[element.id]
-            get_undo_manager().add_undo_action(_UndoRemoveAction(self, element))
+            #get_undo_manager().add_undo_action(_UndoRemoveAction(self, element))
             self.notify(element, 'remove')
+            component.handle(ElementDeleteEvent(self, element))
 #        elif pspec == '__relink__' and not self._elements.has_key(element.id):
 #            log.debug('Relinking element: %s' % element)
 #            self._elements[element.id] = element
 #            self.notify(element, 'create')
 
+component.provideUtility(ElementFactory(), IService, "ElementFactory")
+
+
+@component.adapter(IElementCreateEvent)
+def undo_create_event(event):
+    factory = event.service
+    element = event.element
+    def _undo_create_event():
+        try:
+            del factory._elements[element.id]
+        except KeyError:
+            pass # Key was probably already removed in an unlink call
+        factory.notify(element, 'remove')
+        component.handle(ElementDeleteEvent(factory, element))
+    get_undo_manager().add_undo_action(_undo_create_event)
+
+component.provideHandler(undo_create_event)
+
+
+@component.adapter(IElementDeleteEvent)
+def undo_delete_event(event):
+    factory = event.service
+    element = event.element
+    def _undo_delete_event():
+        factory._elements[element.id] = element
+        factory.notify(element, 'create')
+        component.handle(ElementCreateEvent(factory, element))
+    get_undo_manager().add_undo_action(_undo_delete_event)
+
+component.provideHandler(undo_delete_event)
+
+
+# vim:sw=4:et
