@@ -5,16 +5,17 @@ Classes for grouping different elements.
 import itertools
 
 import gobject
-import diacanvas
 
-class GroupBase(diacanvas.CanvasGroupable):
+# TODO: Remove alltogether (no longer used for Gaphas)
+#class GroupBase(diacanvas.CanvasGroupable):
+class GroupBase(object):
     """
     This class allows to group different diagram elememnts. It should be
     set as a base class of main diagram element. Main diagram element
     (parent) has children.
     """
     def __init__(self):
-        diacanvas.CanvasGroupable.__init__(self)
+        #diacanvas.CanvasGroupable.__init__(self)
         self.children = set()
 
 
@@ -49,15 +50,3 @@ class GroupBase(diacanvas.CanvasGroupable):
 
     def on_groupable_iter(self):
         return iter(self.children)
-
-
-
-class Groupable(gobject.GObjectMeta):
-    """
-    Metaclass for groupable diagram item classes.
-    """
-    def __new__(cls, name, bases, data):
-        c = gobject.GObjectMeta.__new__(cls, name, bases, data)
-        gobject.type_register(c)
-        diacanvas.set_groupable(c)
-        return c
