@@ -74,8 +74,6 @@ class Action(gobject.GObject):
     def execute(self):
         pass
 
-gobject.type_register(Action)
-
 
 class CheckAction(Action):
     """CheckActions can be turned on and off.
@@ -106,8 +104,6 @@ class CheckAction(Action):
         else:
             return Action.do_get_property(self, pspec)
 
-gobject.type_register(CheckAction)
-
 
 class RadioAction(CheckAction):
     """RadioAction is a CheckAction, of which only one action is activated at
@@ -116,8 +112,6 @@ class RadioAction(CheckAction):
     group = None
     
 
-gobject.type_register(RadioAction)
-
 
 class ObjectAction(Action):
     """An ActionObject allows you to create objects in stead of the classes
@@ -172,8 +166,6 @@ class DynamicMenu(gobject.GObject):
         """
         return ()
 
-gobject.type_register(DynamicMenu)
-
 
 class SlotMenu(DynamicMenu):
 
@@ -191,8 +183,6 @@ class SlotMenu(DynamicMenu):
         #print 'menu = ', self._menu
         return self._menu
 
-gobject.type_register(SlotMenu)
-
 
 _no_default = object()
 
