@@ -5,22 +5,21 @@ import gtk
 
 from gaphor import UML
 from gaphor import resource
-from gaphor.diagram.itemtool import ItemTool
+from gaphor.i18n import _
+from gaphor.diagram.interfaces import IPopupMenu
 from gaphor.diagram import get_diagram_item
 from gaphor.undomanager import get_undo_manager
 from gaphor.ui.diagramview import DiagramView
 from gaphor.ui.abstractwindow import AbstractWindow
 
-import gaphor.ui.diagramactions
-import gaphor.diagram.placementactions
-
-from gaphor.event import DiagramItemFocused
+from event import DiagramItemFocused
 from zope import component
 
 class DiagramTab(object):
     
     def __init__(self, owning_window):
         self.diagram = None
+        #self.view = None
         self.owning_window = owning_window
 
     def get_diagram(self):
@@ -34,64 +33,56 @@ class DiagramTab(object):
 
     def set_diagram(self, diagram):
         if self.diagram:
-            self.diagram.disconnect(self.__on_diagram_event)
+            self.diagram.disconnect(self._on_diagram_event)
             #self.diagram.canvas.disconnect(self.__undo_id)
             #self.diagram.canvas.disconnect(self.__snap_to_grid_id)
         self.diagram = diagram
         if diagram:
-            #log.info('set diagram')
-            diagram.canvas.set_property ('allow_undo', 1)
-            diagram.connect(('name', '__unlink__'), self.__on_diagram_event)
-            #self.__undo_id = diagram.canvas.undo_manager.connect('begin_transaction', self.__on_diagram_undo)
-            # Set capabilities:
-            #self.__on_diagram_undo(diagram.canvas.undo_manager)
+            diagram.connect(('name', '__unlink__'), self._on_diagram_event)
 
             if hasattr(self, 'view'):
-                self.view.get_hadjustment().set_value(0.0)
-                self.view.get_vadjustment().set_value(0.0)
+                self.view.hadjustment.set_value(0.0)
+                self.view.vadjustment.set_value(0.0)
 
 
     def construct(self):
-        title = self.diagram and self.diagram.name or '<None>'
+        title = self.diagram and self.diagram.name or _('<None>')
 
         table = gtk.Table(2,2, False)
-        #table.set_row_spacings (4)
-        #table.set_col_spacings (4)
+        #table.set_row_spacings(4)
+        #table.set_col_spacings(4)
 
         frame = gtk.Frame()
-        frame.set_shadow_type (gtk.SHADOW_IN)
-        table.attach (frame, 0, 1, 0, 1,
-                      gtk.EXPAND | gtk.FILL | gtk.SHRINK,
-                      gtk.EXPAND | gtk.FILL | gtk.SHRINK)
+        frame.set_shadow_type(gtk.SHADOW_IN)
+        table.attach(frame, 0, 1, 0, 1,
+                     gtk.EXPAND | gtk.FILL | gtk.SHRINK,
+                     gtk.EXPAND | gtk.FILL | gtk.SHRINK)
 
-        view = DiagramView (diagram=self.diagram)
-        view.set_scroll_region(0, 0, 600, 450)
+        view = DiagramView(diagram=self.diagram)
+        #view.set_scroll_region(0, 0, 600, 450)
 
-        frame.add (view)
+        frame.add(view)
         
-        sbar = gtk.VScrollbar(view.get_vadjustment())
-        table.attach (sbar, 1, 2, 0, 1, gtk.FILL,
-                      gtk.EXPAND | gtk.FILL | gtk.SHRINK)
+        sbar = gtk.VScrollbar(view.vadjustment)
+        table.attach(sbar, 1, 2, 0, 1, gtk.FILL,
+                     gtk.EXPAND | gtk.FILL | gtk.SHRINK)
 
-        sbar = gtk.HScrollbar(view.get_hadjustment())
-        table.attach (sbar, 0, 1, 1, 2, gtk.EXPAND | gtk.FILL | gtk.SHRINK,
-                      gtk.FILL)
+        sbar = gtk.HScrollbar(view.hadjustment)
+        table.attach(sbar, 0, 1, 1, 2, gtk.EXPAND | gtk.FILL | gtk.SHRINK,
+                     gtk.FILL)
 
-        view.connect('notify::tool', self.__on_view_notify_tool)
-        view.connect_after('event-after', self.__on_view_event_after)
-        view.connect('focus-item', self.__on_view_focus_item)
-        view.connect('select-item', self.__on_view_select_item)
-        view.connect('unselect-item', self.__on_view_select_item)
-        view.connect_after('key-press-event', self.__on_key_press_event)
-        view.connect('drag_data_received', self.__on_drag_data_received)
+        #view.connect('notify::tool', self._on_view_notify_tool)
+        view.connect_after('event-after', self._on_view_event_after)
+        view.connect('focus-changed', self._on_view_focus_changed)
+        view.connect('selection-changed', self._on_view_selection_changed)
+        #view.connect('unselect-item', self._on_view_selection_changed)
+        view.connect_after('key-press-event', self._on_key_press_event)
+        view.connect('drag-data-received', self._on_drag_data_received)
+
+        #item_tool = ItemTool(self.owning_window.get_action_pool())
+        #view.get_default_tool().set_item_tool(item_tool)
         self.view = view
 
-        item_tool = ItemTool(self.owning_window.get_action_pool())
-        view.get_default_tool().set_item_tool(item_tool)
-
-        view.get_hadjustment().set_value(0.0)
-        view.get_vadjustment().set_value(0.0)
-
         table.show_all()
 
         self.owning_window.new_tab(self, table, title)
@@ -108,7 +99,7 @@ class DiagramTab(object):
         del self.view
         del self.diagram
 
-    def __on_key_press_event(self, view, event):
+    def _on_key_press_event(self, view, event):
         """Handle the 'Delete' key. This can not be handled directly (through
         GTK's accelerators) since otherwise this key will confuse the text
         edit stuff.
@@ -118,16 +109,15 @@ class DiagramTab(object):
                 self.owning_window.execute_action('EditDelete')
 
 
-    def __on_view_event_after(self, view, event):
+    def _on_view_event_after(self, view, event):
         # handle mouse button 3 (popup menu):
         if event.type == gtk.gdk.BUTTON_PRESS:
             # First push the undo stack...
             #view.canvas.push_undo(None)
             pass
         if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
-            vitem = view.focus_item
-            if vitem:
-                item = vitem.item
+            item = view.focused_item
+            if item:
                 popup_menu = item.get_popup_menu()
                 if popup_menu:
                     self.owning_window._construct_popup_menu(menu_def=popup_menu,
@@ -135,21 +125,17 @@ class DiagramTab(object):
                     return True
         return False
 
-    def __on_view_focus_item(self, view, focus_item):
+    def _on_view_focus_changed(self, view, focus_item):
         self.owning_window.execute_action('ItemFocus')
         component.handle(DiagramItemFocused(focus_item))
 
-    def __on_view_select_item(self, view, select_item):
+    def _on_view_selection_changed(self, view, selection):
         self.owning_window.execute_action('ItemSelect')
 
-    def __on_view_notify_tool(self, view, pspec):
+    def _on_view_notify_tool(self, view, pspec):
         self.owning_window.execute_action('ToolChange')
 
-#    def __on_diagram_undo(self, undo_manager):
-#        #log.info('set undo stack %s' % (undo_manager))
-#        self.owning_window.execute_action('editundostack')
-
-    def __on_diagram_event(self, element, pspec):
+    def _on_diagram_event(self, element, pspec):
         if pspec == '__unlink__':
             self.close()
         elif pspec.name == 'name':
@@ -157,7 +143,9 @@ class DiagramTab(object):
             self.owning_window.set_tab_label(self, element.name)
             #self.get_window().set_title(self.diagram.name or '<None>')
 
-    def __on_drag_data_received(self, view, context, x, y, data, info, time):
+    def _on_drag_data_received(self, view, context, x, y, data, info, time):
+        """Handle data dropped on the canvas.
+        """
         #print 'drag_data_received'
         if data and data.format == 8 and info == DiagramView.TARGET_ELEMENT_ID:
             #print 'drag_data_received:', data.data, info
@@ -174,20 +162,20 @@ class DiagramTab(object):
                 get_undo_manager().begin_transaction()
                 item = self.diagram.create(item_class)
                 assert item
-                wx, wy = view.window_to_world(x + view.get_hadjustment().value,
-                                              y + view.get_vadjustment().value)
+                wx, wy = view.transform_point_c2w(x + view.hadjustment.value,
+                                              y + view.vadjustment.value)
 
-                ix, iy = item.affine_point_w2i(max(0, wx), max(0, wy))
-                item.move(ix, iy)
-                item.set_property ('subject', element)
+                ix, iy = view.canvas.get_matrix_w2i(item, calculate=True).transform_point(max(0, wx), max(0, wy))
+                item.matrix.translate(ix, iy)
+                item.subject = element
                 get_undo_manager().commit_transaction()
                 view.unselect_all()
-                view.focus(view.find_view_item(item))
+                view.focused_item = item
 
                 self.owning_window.execute_action('ItemDiagramDrop')
 
             else:
-                log.warning ('No graphical representation for UML element %s' % type(element).__name__)
+                log.warning('No graphical representation for UML element %s' % type(element).__name__)
             context.finish(True, False, time)
         else:
             context.finish(False, False, time)
