@@ -5,7 +5,7 @@ import gtk
 
 from gaphor import resource
 from gaphor import UML
-from gaphor import undomanager
+from gaphor.undomanager import get_undo_manager
 from gaphor.i18n import _
 from gaphor.ui import namespace
 from gaphor.ui.abstractwindow import AbstractWindow
@@ -13,12 +13,11 @@ from gaphor.ui.diagramtab import DiagramTab
 from gaphor.ui.toolbox import Toolbox
 from gaphor.ui.menufactory import toolbox_to_menu
 
-# Load actions
-from gaphor.ui import mainactions, diagramactions
 from gaphor.ui.objectinspector import ObjectInspector
 
 
-from gaphor.interfaces import *
+from interfaces import IDiagramElementReceivedFocus
+from gaphor.interfaces import IServiceEvent
 from zope import component
 
 class MainWindow(AbstractWindow):
@@ -111,9 +110,9 @@ class MainWindow(AbstractWindow):
                 'ViewZoomOut',
                 'ViewZoom100',
                 'separator',
-                'SnapToGrid',
-                'ShowGrid',
-                'separator',
+                #'SnapToGrid',
+                #'ShowGrid',
+                #'separator',
                 'CreateDiagram',
                 'DeleteDiagram',
                 'separator',
@@ -154,8 +153,8 @@ class MainWindow(AbstractWindow):
 
     def __init__(self):
         AbstractWindow.__init__(self)
-        self.__filename = None
-        #self.__transient_windows = []
+        self._filename = None
+        #self._transient_windows = []
         self.notebook_map = {}
 
     def get_model(self):
@@ -174,7 +173,7 @@ class MainWindow(AbstractWindow):
     def set_filename(self, filename):
         """Set the file name of the currently opened model.
         """
-        self.__filename = filename
+        self._filename = filename
 
         # Add to recent files list
         if filename:
@@ -187,12 +186,12 @@ class MainWindow(AbstractWindow):
     def get_filename(self):
         """Return the file name of the currently opened model.
         """
-        return self.__filename
+        return self._filename
 
 #    def get_transient_windows(self):
 #        """Get the windows that act as child windows of the main window.
 #        """
-#        return self.__transient_windows
+#        return self._transient_windows
 
     def get_current_diagram_tab(self):
         """Get the currently opened and viewed DiagramTab, shown on the right
@@ -228,8 +227,8 @@ class MainWindow(AbstractWindow):
         return answer == gtk.RESPONSE_YES
 
     def show_diagram(self, diagram):
-        """Show a Diagram element in a new tab. If a tab is already open,
-        show that one instead.
+        """Show a Diagram element in a new tab.
+        If a tab is already open, show that one instead.
         """
         # Try to find an existing window/tab and let it get focus:
         for tab in self.get_tabs():
@@ -317,15 +316,15 @@ class MainWindow(AbstractWindow):
         self.window.connect('size-allocate', self.on_window_size_allocate)
 
         # Set some handles for the undo manager
-        undomanager.get_undo_manager().connect('begin_transaction', self.on_undo)
-        undomanager.get_undo_manager().connect('add_undo_action', self.on_undo)
+        #get_undo_manager().connect('begin_transaction', self.on_undo)
+        #get_undo_manager().connect('add_undo_action', self.on_undo)
 
     def add_transient_window(self, window):
         """Add a window as a sub-window of the main application.
         """
         # Assign the window the accelerators od the main window too
         pass #window.get_window().add_accel_group(self.accel_group)
-        #self.__transient_windows.append(window)
+        #self._transient_windows.append(window)
         #window.connect(self.on_transient_window_closed)
 
     # Notebook methods:
@@ -433,6 +432,8 @@ class MainWindow(AbstractWindow):
         """
         self._check_state(AbstractWindow.STATE_ACTIVE)
         self.execute_action('OpenModelElement')
+        # Set the pointer tool as default tool.
+        self.execute_action('Pointer')
 
     def on_view_cursor_changed(self, view):
         """Another row is selected, execute a dummy action.
@@ -456,20 +457,24 @@ class MainWindow(AbstractWindow):
 #        print 'Box', box_name, 'is visible:', visible
 #        #resource.set('ui.toolbox.%s' % box_name, visible, persistent=True)
 
-    def on_undo(self, *args):
-        self.execute_action('UndoStack')
-
 #    def on_transient_window_closed(self, window):
-#        assert window in self.__transient_windows
+#        assert window in self._transient_windows
 #        log.debug('%s closed.' % window)
-#        self.__transient_windows.remove(window)
+#        self._transient_windows.remove(window)
 
-    def __on_transient_window_notify_title(self, window):
+    def _on_transient_window_notify_title(self, window):
         pass
 
-    #def __on_element_factory_signal(self, obj, key):
-        #print '__on_element_factory_signal', key
+    #def _on_element_factory_signal(self, obj, key):
+        #print '_on_element_factory_signal', key
         #factory = resource(UML.ElementFactory)
         #self.set_capability('model', not factory.is_empty())
 
 gtk.accel_map_add_filter('gaphor')
+
+@component.adapter(IServiceEvent)
+def on_undo(*args):
+    resource(MainWindow).execute_action('UndoStack')
+
+component.provideHandler(on_undo)
+
